import 'daterangepicker';
import 'tempusdominus-bootstrap-4';
import type { Moment } from 'moment';

interface DateInput {
    year: HTMLSelectElement;
    month: HTMLSelectElement;
    day: HTMLSelectElement;
    hour: HTMLSelectElement;
    minute: HTMLSelectElement;
}

const displayToast = (window as any).displayToast as (messageType: string, message:string) => void;

const setDateTimeInput = (date: Date, selectorFactory: (property: string) => string) => {
    const input: DateInput = {
        year: document.querySelector(selectorFactory("date_year")),
        month: document.querySelector(selectorFactory("date_month")),
        day: document.querySelector(selectorFactory("date_day")),
        hour: document.querySelector(selectorFactory("time_hour")),
        minute: document.querySelector(selectorFactory("time_minute"))
    }

    input.year.value = date.getUTCFullYear().toString();
    input.month.value = (date.getUTCMonth() + 1).toString();
    input.day.value = date.getUTCDate().toString();
    input.hour.value = date.getUTCHours().toString();
    input.minute.value = date.getUTCMinutes().toString();
}

$(() => {
    ($("#date") as any ).daterangepicker({
        "timePicker": true,
        "timePicker24Hour": true,
        "locale": {
            "format": "YYYY-MM-DD-HH-mm",
            "separator": " → ",
            "applyLabel": "Appliquer",
            "cancelLabel": "Annuler",
            "fromLabel": "De",
            "toLabel": "À",
            "customRangeLabel": "Custom",
            "weekLabel": "S",
            "daysOfWeek": [
                "Di",
                "Lu",
                "Ma",
                "Me",
                "Je",
                "Ve",
                "Sa"
            ],
            "monthNames": [
                "Janvier",
                "Février",
                "Mars",
                "Avril",
                "Mai",
                "Juin",
                "Juillet",
                "Août",
                "Septembre",
                "Octobre",
                "Novembre",
                "Décembre"
            ],
            "firstDay": 0,
        },
    }, (startMoment: Moment, endMoment: Moment) => {
        const start = startMoment.toDate();
        setDateTimeInput(start, (property: string): string => `#session_startDate_${property}`);

        const end = endMoment.toDate();
        setDateTimeInput(end, (property: string): string => `#session_endDate_${property}`);
    });

    $("#sessionAddForm").on('submit', (e) => {
        e.preventDefault();

        const form = e.currentTarget as HTMLFormElement;
        if (!form.checkValidity()) {
            form.classList.add("was-validated");
            return;
        }

        const $form = $(form);
        const submitButton = form.querySelector('#session_submit') as HTMLButtonElement;
        const oldText = submitButton.innerText;

        const spinner = document.createElement("div") as HTMLDivElement;
        spinner.classList.add("spinner-border");
        spinner.setAttribute("role", "status");
        const spinner_sr = document.createElement('span') as HTMLSpanElement;
        spinner_sr.classList.add("sr-only");
        spinner_sr.innerText = "Loading...";

        spinner.appendChild(spinner_sr);
        submitButton.innerHTML = "";
        submitButton.appendChild(spinner);
        submitButton.disabled = true;
        $.ajax({
            url: $form.attr('action'),
            method: $form.attr('method'),
            data: $form.serialize(),
            success: () => {
                displayToast("success", "La session a bien été créée");
                setTimeout(() => location.reload(), 1000);
            },
            error: () => {
                displayToast("error","Une erreur est survenue ! Veuillez réessayer ultérieurement.");
            },
            complete: () => {
                submitButton.disabled = false;
                submitButton.innerText = oldText;
            }
        });
    })
});
