import './datatables/index';
import './datatables/responsive';
import './datatables/buttons';
import language from 'datatables.net-plugins/i18n/fr-FR.mjs';

import * as pdfMake from "pdfmake/build/pdfmake";
import * as pdfFonts from 'pdfmake/build/vfs_fonts';

(<any>pdfMake).vfs = pdfFonts.pdfMake.vfs;
(window as any).JSZip = require('jszip');

$(() => {
    const labelApplicationStatus = JSON.parse(document.querySelector('body')?.dataset.statusesData || '');
    const supportedApplicationStatus = JSON.parse(document.querySelector('body')?.dataset.stats || '');
    let activeStatuses = Array<string>();

    const dataTable = ($("#applications") as any).DataTable({
        language,
        "responsive": true,
        "lengthChange": false,
        "autoWidth": false,
        "buttons": ["copy", "csv", "pdf", "print", "colvis"],
        "columnDefs": [
            { orderSequence: ["desc", "asc"], targets: 'inverse-sort' },
            { orderable: false, targets: 'disable-sort' },
        ],
        "order": [[$('th.default-sort').index(), 'desc']],
    });
    dataTable.buttons().container().appendTo('#applications_wrapper .col-md-6:eq(0)');

    function setFilterTags(activeStatuses: Array<string>): void {
        const buttons = document.querySelectorAll('.filter-btn') as NodeListOf<HTMLButtonElement>;
        buttons.forEach((value: HTMLButtonElement) => {
            const id = value.dataset.id || '';
            const obj = labelApplicationStatus[id];
            const enabled = activeStatuses.includes(id);
            value.classList.toggle('btn-outline-' + obj.badge, !enabled);
            value.classList.toggle('btn-' + obj.badge, enabled);
        });
    }

    function filterByStatuses() {
        setFilterTags(activeStatuses);
        dataTable.column(2).search( '(' + activeStatuses.map((e) => labelApplicationStatus[e].label).join(')|(') + ')', true, false, true).draw();
    }

    function toogleTag(id: string): void {
        if (activeStatuses.includes(id)) {
            activeStatuses = activeStatuses.filter((e) => e != id);
        } else {
            activeStatuses.push(id);
        }
        filterByStatuses();
    }

    document.querySelectorAll('.filter-btn').forEach((value: HTMLButtonElement) => {
        const id = value.dataset.id || '';
        value.addEventListener('click', () => {
            toogleTag(id);
        });
    });

    supportedApplicationStatus.forEach((value: string) => {
        $("#filter-" + value + "-only").on("click", () => {
            activeStatuses = [value];
            filterByStatuses();
        });
    });
});

$(() => {
    const dataTable = ($("#approved") as any).DataTable({
        language,
        "responsive": true,
        "lengthChange": false,
        "autoWidth": false,
        "buttons": [
            "csv",
            {
                extend: "excel",
                autoFilter: true,
                sheetName: "Demandes approuvées",
            },
            "print",
            "colvis",
        ],
    });
    dataTable.buttons().container().appendTo('#approved_wrapper .col-md-6:eq(0)');
});
