// Set a Cookie
function setCookie(cName: string, cValue: string, expDays: number) : void {
  let date = new Date();
  date.setTime(date.getTime() + (expDays * 24 * 60 * 60 * 1000));
  const expires = "expires=" + date.toUTCString();
  document.cookie = cName + "=" + cValue + "; " + expires + "; path=/";
}

// Get Cookie
function getCookie(cName: string) : string {
  const name = cName + "=";
  const cDecoded = decodeURIComponent(document.cookie); //to be careful
  const cArr = cDecoded.split('; ');
  let res;
  cArr.forEach(val => {
    if (val.indexOf(name) === 0) res = val.substring(name.length);
  })
  return res;
}

// Get preferred mode
let darkMode = false;

function toogleDarkMode(force: boolean | null = null) {
  darkMode = (force === null) ? !darkMode : force;
  setCookie("useDarkMode", darkMode ? "true" : "false", 1);

  $("[dark-mode-switch]").each(function () {
    if (this.dataset.darkModeClass) {
      const classes = this.dataset.darkModeClass.split(' ');
      classes.forEach((className: string) => this.classList.toggle(className, darkMode));
    }
    if (this.dataset.lightModeClass) {
      const classes = this.dataset.lightModeClass.split(' ');
      classes.forEach((className: string) => this.classList.toggle(className, !darkMode));
    }

    if (this.dataset.lightModeSrc && this.dataset.darkModeSrc) {
      (this as HTMLImageElement).src = darkMode ? this.dataset.darkModeSrc : this.dataset.lightModeSrc;

    }
  });
}

// Change darkmode on system darkmode change
window.matchMedia("(prefers-color-scheme: dark)").addEventListener("change", (ev: MediaQueryListEvent) => {
  if (ev.matches) {
    toogleDarkMode(true);
  }
});
window.matchMedia("(prefers-color-scheme: light)").addEventListener("change", (ev: MediaQueryListEvent) => {
  if (ev.matches) {
    toogleDarkMode(false);
  }
});

// Toggle dark mode when darkmode button is clicked
$("#dark-button").on("click", () => {
  toogleDarkMode();
});

// Check if cookie is already set, or assume from system preference (defaulting to no darkmode)
let foundDarkMode: boolean = getCookie("useDarkMode") == "true";
if (foundDarkMode === undefined) {
  foundDarkMode = window.matchMedia("(prefers-color-scheme: dark)").matches
} else {
  foundDarkMode = foundDarkMode ;
}
toogleDarkMode(foundDarkMode);
