<?php

namespace App\Command;

use App\Entity\Admin;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\Question;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasher;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;

#[AsCommand(
    name: 'app:admin:user:create',
    description: 'Creates a new admin user',
)]
class AdminUserCreateCommand extends Command
{
    private UserPasswordHasherInterface $passwordHasher;
    private EntityManagerInterface $manager;

    public function __construct(UserPasswordHasherInterface $passwordHasher, EntityManagerInterface $manager)
    {
        parent::__construct();

        $this->passwordHasher = $passwordHasher;
        $this->manager = $manager;
    }

    protected function configure(): void
    {
        $this
            ->addOption('username', 'u', InputArgument::REQUIRED | InputOption::VALUE_REQUIRED, 'Username of the new user');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        $username = $input->getOption('username');

        $helper = $this->getHelper('question');

        $question = new Question("Enter the password: ");
        $question->setHidden(true);

        $password = $helper->ask($input, $output, $question);

        $admin = new Admin($username);
        $admin->setPassword($this->passwordHasher->hashPassword(
            $admin,
            $password
        ));
        $admin->setRoles(["ROLE_ADMIN"]);

        $this->manager->persist($admin);
        $this->manager->flush();

        $io->success("The admin user ${username} has been created!");

        return Command::SUCCESS;
    }
}
