<?php

namespace App\Command;

use App\Repository\ParkingApplicationRepository;
use App\Repository\SessionRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

#[AsCommand(
    name: 'app:fix-addresses',
    description: 'Finds postal addresses with more commas than expected',
)]
class FixAddressesCommand extends Command
{
    private ParkingApplicationRepository $parkingApplicationRepository;
    private SessionRepository $sessionRepository;
    private EntityManagerInterface $manager;

    public function __construct(
        ParkingApplicationRepository $parkingApplicationRepository,
        SessionRepository $sessionRepository,
        EntityManagerInterface $manager,
    ) {
        parent::__construct();

        $this->parkingApplicationRepository = $parkingApplicationRepository;
        $this->sessionRepository = $sessionRepository;
        $this->manager = $manager;
    }

    protected function configure(): void
    {
        $this
            ->addOption('session', 's', InputArgument::REQUIRED | InputOption::VALUE_REQUIRED, 'Session name to check');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);

        $sessionName = $input->getOption('session');
        $session = $this->sessionRepository->findOneBy(["name" => $sessionName]);

        if (!$session)
        {
            $io->error("Session $sessionName not found");
            return Command::INVALID;
        }

        $parkingApps = $this->parkingApplicationRepository->findBy(["session" => $session]);
        foreach ($parkingApps as $parkingApp)
        {
            $parkingApp->splitAddress();
            $this->manager->persist($parkingApp);
        }

        $this->manager->flush();

        return Command::SUCCESS;
    }
}
