<?php

namespace App\Entity;

use App\Repository\FileRepository;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use Doctrine\ORM\Mapping\InheritanceType;
use Doctrine\ORM\Mapping\DiscriminatorColumn;
use Symfony\Component\HttpFoundation\File\UploadedFile;

#[ORM\Entity(repositoryClass: FileRepository::class)]
#[InheritanceType("SINGLE_TABLE")]
#[DiscriminatorColumn(name: "fileType", type: "string")]
class File
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column(type: "integer")]
    private $id;

    #[ORM\Column(type: "string", length: 127)]
    #[Assert\NotBlank()]
    #[Assert\Length(max: 127)]
    private $fileName;

    #[ORM\Column(type: "boolean")]
    #[Assert\NotNull()]
    private $validated = false;

    #[ORM\Column(type: "datetime", nullable: true)]
    #[Assert\NotBlank(allowNull: true)]
    private $validationDate = null;

    #[ORM\Column(type: 'boolean', nullable: false)]
    #[Assert\NotNull()]
    private $deleted = false;

    #[ORM\Column(type: "datetime", nullable: true)]
    #[Assert\NotBlank(allowNull: true)]
    private $deletionDate = null;

    /**
     * @var UploadedFile
     */
    private $file;

    public function __construct(File $file = null)
    {
        if ($file !== null) {
            $this->setFileName($file->getFileName());
            $this->setValidated($file->getValidated());
            $this->setValidationDate($file->getValidationDate());
            $this->setFile($file->getFile());
        }
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getFileName(): ?string
    {
        return $this->fileName;
    }

    public function setFileName(?string $fileName): self
    {
        $this->fileName = $fileName;

        return $this;
    }

    public function getValidated(): ?bool
    {
        return $this->validated;
    }

    public function setValidated(bool $validated): self
    {
        $this->validated = $validated;

        return $this;
    }

    public function getValidationDate(): ?\DateTimeInterface
    {
        return $this->validationDate;
    }

    public function setValidationDate(?\DateTimeInterface $validationDate): self
    {
        $this->validationDate = $validationDate;

        return $this;
    }

    public function validate(): self {
        $this->setValidated(true);
        $this->setValidationDate(new \DateTime('now', new \DatetimeZone('America/Toronto')));
        return $this;
    }

    public function getFile(): ?UploadedFile
    {
        return $this->file;
    }

    public function setFile(?UploadedFile $file): self
    {
        $this->file = $file;

        return $this;
    }

    public function getDeleted(): ?bool
    {
        return $this->deleted;
    }

    public function setDeleted(bool $deleted): self
    {
        $this->deleted = $deleted;
        return $this;
    }

    public function delete(): self
    {
        // TODO: Delete File
        $this->setDeleted(true);
        $this->setDeletionDate(new \DateTime('now', new \DatetimeZone('America/Toronto')));

        return $this;
    }

    public function getDeletionDate(): ?\DateTimeInterface
    {
        return $this->deletionDate;
    }

    public function setDeletionDate(?\DateTimeInterface $deletionDate): self
    {
        $this->deletionDate = $deletionDate;

        return $this;
    }

    public function __serialize()
    {
        return array(
            $this->id,
            $this->fileName,
            $this->validated,
            $this->validationDate,
            $this->deleted,
            $this->deletionDate
        );
    }

    public function __unserialize(array $serialized)
    {
        list (
            $this->id,
            $this->fileName,
            $this->validated,
            $this->validationDate,
            $this->deleted,
            $this->deletionDate
        ) = $serialized;
    }

    public function getApplication(): ?ParkingApplication
    {
        return null;
    }

    public function setApplication(?ParkingApplication $application): self
    {
        return $this;
    }
}
