<?php

namespace App\Entity;

use App\Repository\SessionRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

#[ORM\Entity(repositoryClass: SessionRepository::class)]
#[UniqueEntity(
    fields: "name",
    message: "session.name.unique"
)]
#[UniqueEntity(
    fields: ["startDate", "endDate"],
    message: "session.dates.unique",
    repositoryMethod: "findByDateIntersection"
)]
#[UniqueEntity(
    fields: "archived",
    message: "session.active.unique",
    repositoryMethod: "findActiveOrNothing"
)]
class Session
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column(type: 'integer')]
    private $id;

    #[ORM\Column(type: 'string', length: 30, unique: true)]
    private $name;

    #[ORM\Column(type: 'datetime')]
    private $startDate;

    #[ORM\Column(type: 'datetime')]
    private $endDate;

    #[ORM\Column(type: 'boolean')]
    private $archived = false;

    #[ORM\OneToMany(mappedBy: 'session', targetEntity: ParkingApplication::class, orphanRemoval: true)]
    private $parkingApplications;

    public function __construct()
    {
        $this->parkingApplications = new ArrayCollection();
        $this->machinchose = new ArrayCollection();
    }

    #[Assert\Callback]
    public function validate(ExecutionContextInterface $context, $payload)
    {
        if ($this->getEndDate() <= $this->getStartDate()) {
            $context->buildViolation("session.dates.end-after-start")
                ->addViolation();
        }
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getStartDate(): ?\DateTimeInterface
    {
        return $this->startDate;
    }

    public function setStartDate(\DateTimeInterface $startDate): self
    {
        $this->startDate = $startDate;

        return $this;
    }

    public function getEndDate(): ?\DateTimeInterface
    {
        return $this->endDate;
    }

    public function setEndDate(\DateTimeInterface $endDate): self
    {
        $this->endDate = $endDate;

        return $this;
    }

    public function getArchived(): ?bool
    {
        return $this->archived;
    }

    public function setArchived(bool $archived): self
    {
        $this->archived = $archived;

        return $this;
    }

    /**
     * @return Collection|ParkingApplication[]
     */
    public function getParkingApplications(): Collection
    {
        return $this->parkingApplications;
    }

    public function addParkingApplication(ParkingApplication $parkingApplication): self
    {
        if (!$this->parkingApplications->contains($parkingApplication)) {
            $this->parkingApplications[] = $parkingApplication;
            $parkingApplication->setSession($this);
        }

        return $this;
    }

    public function removeParkingApplication(ParkingApplication $parkingApplication): self
    {
        if ($this->parkingApplications->removeElement($parkingApplication)) {
            // set the owning side to null (unless already changed)
            if ($parkingApplication->getSession() === $this) {
                $parkingApplication->setSession(null);
            }
        }

        return $this;
    }
}
