<?php

namespace App\Entity;

use App\Enum\VehicleTypeEnum;
use App\Repository\VehicleRepository;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

#[ORM\Entity(repositoryClass: VehicleRepository::class)]
class Vehicle
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column(type: "integer")]
    private $id;

    #[ORM\Column(type: "string", length: 255)]
    #[Assert\NotBlank()]
    #[Assert\Length(max: 255)]
    private $manufacturer;

    #[ORM\Column(type: "string", length: 255)]
    #[Assert\NotBlank()]
    #[Assert\Length(max: 255)]
    private $model;

    #[ORM\Column(type: "string", length: 31)]
    #[Assert\NotBlank()]
    #[Assert\Length(max: 31)]
    private $colour;

    #[ORM\Column(type: "integer")]
    #[Assert\NotNull()]
    #[Assert\LessThanOrEqual(2100)]
    #[Assert\GreaterThan(1980)]
    private $year;

    #[ORM\Column(type: "string", length: 31)]
    #[Assert\NotBlank()]
    #[Assert\Length(max: 31)]
    private $licensePlate;

    #[ORM\Column]
    #[Assert\NotBlank()]
    private VehicleTypeEnum $type;

    #[ORM\ManyToOne(targetEntity: ParkingApplication::class, inversedBy: "vehicles")]
    #[ORM\JoinColumn(nullable: false)]
    #[Assert\NotNull()]
    private $application;

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getManufacturer(): ?string
    {
        return $this->manufacturer;
    }

    public function setManufacturer(string $manufacturer): self
    {
        $this->manufacturer = $manufacturer;

        return $this;
    }

    public function getModel(): ?string
    {
        return $this->model;
    }

    public function setModel(string $model): self
    {
        $this->model = $model;

        return $this;
    }

    public function getColour(): ?string
    {
        return $this->colour;
    }

    public function setColour(string $colour): self
    {
        $this->colour = $colour;

        return $this;
    }

    public function getYear(): ?int
    {
        return $this->year;
    }

    public function setYear(int $year): self
    {
        $this->year = $year;

        return $this;
    }

    public function getLicensePlate(): ?string
    {
        return $this->licensePlate;
    }

    public function setLicensePlate(string $licensePlate): self
    {
        $this->licensePlate = $licensePlate;

        return $this;
    }

    public function getType(): ?VehicleTypeEnum
    {
        return $this->type;
    }

    public function setType(VehicleTypeEnum $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getApplication(): ?ParkingApplication
    {
        return $this->application;
    }

    public function setApplication(?ParkingApplication $application): self
    {
        $this->application = $application;

        return $this;
    }
}
