<?php

namespace App\Form;

use App\Entity\ParkingApplication;
use App\Enum\AssociationEnum;
use App\Form\FileType as CustomFileType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\CollectionType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\EnumType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\Extension\Core\Type\TelType;
use Symfony\Component\Validator\Constraints\Length;

class ParkingApplicationType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add('forename', TextType::class, ['label' => 'parking-app.forename', 'help' => 'parking-app.help.forename'])
            ->add('surname', TextType::class, ['label' => 'parking-app.surname', 'help' => 'parking-app.help.surname'])
            ->add('email', EmailType::class, ['label' => 'parking-app.email', 'help' => 'parking-app.help.email', 'attr' => ['placeholder'=>'parking-app.placeholder.email', /*'pattern'=> '[a-zA-Z0-9\-_]+.[a-zA-Z0-9\-_]+@polymtl\.ca'*/]])
            ->add('street', null, ['label' => 'parking-app.street', 'help' => 'parking-app.help.street'])
            ->add('city', null, ['label' => 'parking-app.city'])
            ->add('postalCode', null, ['label' => 'parking-app.postal-code'])
            ->add('province', ChoiceType::class, ['label' => 'parking-app.province.label', 'choices' => ['parking-app.province.choices.qc' => 'QC', 'parking-app.province.choices.on' => 'ON', 'parking-app.province.choices.vt' => 'VT'], 'data' => 'QC' ])
            ->add('country', ChoiceType::class, ['label' => 'parking-app.country.label', 'choices' => ['parking-app.country.choices.can' => 'Canada', 'parking-app.country.choices.usa' => 'États-Unis'], 'data' => 'Canada' ])
            ->add('studentId', TextType::class, ['label' => 'parking-app.student-id', 'help' => 'parking-app.help.student-id', 'attr' => ['placeholder' => 'parking-app.placeholder.student-id', 'minlength' => '7', 'maxlength' => '7', 'pattern'=> '[0-9]{7}']])
            ->add('phone', TelType::class, ['label' => 'parking-app.phone', 'help' => 'parking-app.help.phone', 'attr' => ['placeholder' => 'parking-app.placeholder.phone', 'minlength' => '3', 'pattern'=> '^[\+]?[(]?[0-9]{3}[)]?[-\s\.]?[0-9]{3}[-\s\.]?[0-9]{4,6}$']])
            ->add('vehicles', CollectionType::class, [
                'label' => 'parking-app.vehicles',
                'block_name' => 'vehicles',
                'by_reference' => false,
                'entry_type' => VehicleType::class,
                'entry_options' => ['label' => false],
                'allow_add' => true,
                'allow_delete' => true,
                'help' => 'parking-app.help.vehicles'
            ])
            ->add('addressProof', CollectionType::class, [
                'label' => 'parking-app.address-proof',
                'by_reference' => false,
                'entry_type' => CustomFileType::class,
                'entry_options' => ['label' => false],
                'allow_add' => true,
                'allow_delete' => true,
                'help' => 'parking-app.help.address-proof'
            ])
            ->add('carpooling', CheckboxType::class, ['label' => 'parking-app.carpooling', 'required' => false, 'value' => '0', 'help' => 'parking-app.help.carpooling'])
            ->add('carpoolDescription', TextareaType::class, [
                'label' => 'parking-app.carpool-description',
                'required' => false,
                'help' => 'parking-app.help.carpool-description',
                'constraints' => new Length(['max' => 255]),
                'attr' => ['maxlength' => '255']
            ])
            ->add('parent', CheckboxType::class, ['label' => 'parking-app.parent', 'required' => false, 'help' => 'parking-app.help.parent'])
            ->add('parentProof', CollectionType::class, [
                'label' => 'parking-app.parent-proof',
                'by_reference' => false,
                'entry_type' => CustomFileType::class,
                'entry_options' => ['label' => false],
                'allow_add' => true,
                'allow_delete' => true,
                'help' => 'parking-app.help.parent-proof',
            ])
            ->add('association', EnumType::class, [
                'label' => 'parking-app.association.label',
                'class' => AssociationEnum::class,
                'choice_label' => function (AssociationEnum $choice, int $key, string $value) {
                    return "parking-app.association." . $value;
                },
                'help' => 'parking-app.help.association'
            ])
            ->add('physicalCondition', CheckboxType::class, ['label' => 'parking-app.physical-condition', 'required' => false, 'help' => 'parking-app.help.physical-condition'])
            ->add('physicalConditionProof', CollectionType::class, [
                'label' => 'parking-app.physical-condition-proof',
                'by_reference' => false,
                'entry_type' => CustomFileType::class,
                'entry_options' => ['label' => false],
                'allow_add' => true,
                'allow_delete' => true,
                'help' => 'parking-app.help.physical-condition-proof'
            ])
            ->add('submit', SubmitType::class, ['label' => 'button.submit']);
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => ParkingApplication::class,
        ]);
    }
}
