<?php

namespace App\Form;

use App\Entity\Vehicle;
use App\Enum\VehicleTypeEnum;
use App\Enum\VehicleTypeEnum as EnumVehicleTypeEnum;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\IntegerType;
use Symfony\Component\Form\Extension\Core\Type\EnumType;

class VehicleType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add('manufacturer', TextType::class, [
                'label' => 'car.manufacturer.label',
                'attr' => ['placeholder' => 'car.manufacturer.placeholder'],
                'help' => 'car.help.manufacturer',
            ])
            ->add('model', TextType::class, [
                'label' => 'car.model.label',
                'attr' => ['placeholder' => 'car.model.placeholder'],
                'help' => 'car.help.model',
            ])
            ->add('colour', TextType::class, [
                'label' => 'car.colour.label',
                'attr' => ['placeholder' => 'car.colour.placeholder'],
                'help' => 'car.help.colour',
            ])
            ->add('year', TextType::class, [
                'label' => 'car.year.label',
                'attr' => ['placeholder' => 'car.year.placeholder', 'minlength'=>'4', 'maxlength' => '4', 'pattern'=> '(20[0-9]{2})|(19[8-9][0-9])'],
                'help' => 'car.help.year',
            ])
            ->add('licensePlate', TextType::class, [
                'label' => 'car.plate.label',
                'attr' => ['placeholder' => 'car.plate.placeholder'],
                'help' => 'car.help.plate',
            ])
            ->add('type', EnumType::class, [
                'label' => 'car.type.label',
                'class' => VehicleTypeEnum::class,
                'choice_label' => function (VehicleTypeEnum $choice, int $key, string $value) {
                    return "car.type." . $value;
                },
                'help' => 'car.help.type',
            ])
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => Vehicle::class,
        ]);
    }
}
