<?php

namespace App\Service;

use App\Entity\EmailAccessToken;
use App\Entity\ParkingApplication;
use App\Repository\EmailAccessTokenRepository;
use Doctrine\ORM\EntityManagerInterface;

class EmailTokenService
{
    private EntityManagerInterface $entityManager;
    private EmailAccessTokenRepository $tokenRepository;

    public function __construct(
        EntityManagerInterface $entityManager,
        EmailAccessTokenRepository $tokenRepository
    ) {
        $this->entityManager = $entityManager;
        $this->tokenRepository = $tokenRepository;
    }

    public function generate(ParkingApplication $parkingApp): EmailAccessToken
    {
        $emailAccessToken = new EmailAccessToken($parkingApp);
        $this->entityManager->persist($emailAccessToken);
        $this->entityManager->flush();

        return $emailAccessToken;
    }

    public function deleteExpired()
    {
        $expiredTokens = $this->tokenRepository->findExpired();
        $tokenCount = count($expiredTokens);

        foreach ($expiredTokens as $token) {
            $this->entityManager->remove($token);
        }
        $this->entityManager->flush();

        return $tokenCount;
    }
}
