<?php

namespace App\Service;

use App\Entity\File;
use BadMethodCallException;
use Symfony\Component\Form\FormError;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Symfony\Component\String\Slugger\SluggerInterface;
use Symfony\Component\Validator\Validator\ValidatorInterface;

class FileManager
{
    private string $uploadsDir;
    private SluggerInterface $slugger;
    private ValidatorInterface $validator;

    public function __construct(SluggerInterface $slugger, string $uploadsDir, ValidatorInterface $validator)
    {
        $this->uploadsDir = $uploadsDir;
        $this->slugger = $slugger;
        $this->validator = $validator;
    }

    /**
     * @param Collection|File $fileCollection
     */
    public function uploadFileCollection($fileCollection)
    {
        /** @var File $fileEntity */
        foreach ($fileCollection as $fileEntity) {
            $uploadedFile = $fileEntity->getFile();

            if($fileEntity->getFileName() !== NULL) {
                // The file is already uploaded
                if ($uploadedFile !== NULL) {
                    return new FormError("Can't upload over existing file");
                }
                continue;
            }
            else if ($uploadedFile === NULL) {
                return new FormError("File was not uploaded");
            }

            $fileName = $this->uploadFile($uploadedFile);
            if ($fileName === null) return new FormError("file.error");

            $fileEntity->setFileName($fileName);
            $errors = $this->validator->validate($fileEntity);
            if (count($errors) > 0) {
                $fullString = "";
                foreach ($errors as $e) {
                    $fullString .= (string) $e . "\n";
                }
                return new FormError($fullString);
            }
        }
    }

    /** @var UploadedFile $file */
    public function uploadFile(UploadedFile $file): ?string
    {
        $originalFilename = $file->getClientOriginalName();
        $safeFileName = $this->slugger->slug($originalFilename);

        do {
            $newFilename = $safeFileName.'-'.uniqid().'.'.$file->guessExtension();
        } while (file_exists($this->uploadsDir."/".$newFilename));

        $file->move(
            $this->uploadsDir,
            $newFilename
        );

        return $newFilename;
    }
}
